/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "turn.h"

pokerturn_t pokerTurnGet(poker_t *poker, uint8_t playerIndex) {
  pokerplayer_t *player;
  player = poker->players + playerIndex;

  // Can the player do anything?
  if(player->state & (POKER_PLAYER_STATE_FOLDED | POKER_PLAYER_STATE_OUT)) {
    return pokerTurnOut(poker, playerIndex);
  }

  if(pokerTurnCanPlayerCheck(poker, playerIndex)) {
    return pokerTurnCheck(poker, playerIndex);
  }

  return pokerTurnCall(poker, playerIndex);
}

void pokerTurnAction(poker_t *poker, pokerplayer_t *player, pokerturn_t *turn) {
  switch(turn->type) {
    case POKER_TURN_TYPE_BET:
    case POKER_TURN_TYPE_CALL:
    case POKER_TURN_TYPE_ALL_IN:
      pokerBetPlayer(&poker->bet, player, turn->chips);
      break;

    case POKER_TURN_TYPE_CHECK:
      pokerBetPlayer(&poker->bet, player, 0);
      break;

    case POKER_TURN_TYPE_FOLD:
      player->state |= POKER_PLAYER_STATE_FOLDED;
      break;
  }
  
  player->state |= POKER_PLAYER_STATE_ROUND_MOVE;
}

pokerturn_t pokerTurnOut(poker_t *poker, uint8_t player) {
  pokerturn_t turn;
  turn.type = POKER_TURN_TYPE_OUT;
  return turn;
}

pokerturn_t pokerTurnFold(poker_t *poker, uint8_t player) {
  pokerturn_t turn;
  turn.type = POKER_TURN_TYPE_FOLD;
  return turn;
}

pokerturn_t pokerTurnCheck(poker_t *poker, uint8_t player) {
  pokerturn_t turn;
  turn.type = POKER_TURN_TYPE_CHECK;
  return turn;
}

pokerturn_t pokerTurnCall(poker_t *poker, uint8_t playerIndex) {
  pokerturn_t turn;
  pokerplayer_t *player;
  player = poker->players + playerIndex;
  turn = pokerTurnRaise(
    poker, playerIndex, poker->bet.currentBet - player->currentBet
  );
  turn.type = (
    turn.type == POKER_TURN_TYPE_ALL_IN ?
    POKER_TURN_TYPE_CALL_ALL_IN :
    POKER_TURN_TYPE_CALL
  );
  return turn;
}

pokerturn_t pokerTurnRaise(poker_t *poker, uint8_t playerIndex, int32_t chips) {
  pokerturn_t turn;
  pokerplayer_t *player;
  player = poker->players + playerIndex;

  if(player->chips <= chips) {
    turn.chips = player->chips;
    turn.type = POKER_TURN_TYPE_ALL_IN;
  } else {
    turn.chips = chips;
    turn.type = POKER_TURN_TYPE_BET;
  }

  return turn;
}

bool pokerTurnCanPlayerCheck(poker_t *poker, uint8_t playerIndex) {
  return (poker->players + playerIndex)->currentBet >= poker->bet.currentBet;
}